#include <pcap.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h> //to import getopt library
#include <set>
//#include "boost/program_options.hpp"


#include "../header/config.h"
#include "../header/timing.h"
#include "../header/util.h"
#include "../header/print_pkt.h"
#include "../header/connection_class.h"
#include "sdhash/sdbf/sdbf_class.h"
#include "sdhash/sdbf/sdbf_defines.h"
#include "mrsh/header/fingerprintList.h"
#include "mrsh/header/main.h"
#include "mrsh/header/config.h"

using namespace std;

sniffer_parameters global_param = {
    0,  //verbose_flag
    0,  //output_flag
    0,  //reverse_flag
    0,  //hash_list_flag
    0,  //statistics_flag
    1,  //mrsh_flag , default
    0,  //sdhash_flag
    0,  //offline flag
    0,  //traffic_file_flag
    0,  // mutliple packet analysis

    0,    //tcp/ip layer of operation
    MRSH, //algorithm to use
    "", //output_file
    "", //hash_list_name
    "", //dump_filename
    "", //reverse_filename
    "", //write_packet_filename
    "lo" //interface,default to loopback
};

/*hack to avoid compilation error*/
const char *write_packet_filename = global_param.write_packet_filename.c_str(); 

/*global variables */
/*pcap variables, declared global so that freeing memory on ctrl-c is easy*/
pcap_t *handle;             /**< handle to access packets using pcap*/
struct bpf_program *fp;

/*statistics */
long total_packet_size; /**< cumulative size of all the packets processed*/
long total_header_length; /**< cumulative header length of all the packets processed*/
long total_payload_length; /**< total payload length of packets in one session */ 
long total_ip_header_size; /**< total ip header length of packets in one session */
long total_tcp_header_size; /**< total tcp header of packets in one session */
long packet_count = 0;      /**< counter for total packets processed in one session */
double total_time = 0.0;    /**< total time spent in processing packets in one session */ 
long min_packet_size=100000; /**< minimum packet size processed in one session */ 
long max_packet_size=0;      /**< maximum packet size  processed in one session */

/*Global for SDBF*/
sdbf_set *set_original; /**< sdbf_set containing hashes of original files */
sdbf_set *reverse_set_original;  /**< sdbf_set containing hash for the packet under process */ 

/*Global for MRSH*/
FINGERPRINT_LIST *fpl;  /**< bloom array for mrsh containing list of hashes of original files */
MODES *mode; 
double mrsh_total_score = 0.0;
set<connection> mrsh_multiple_packet_set;

/*stream global variables*/
std::ofstream fileStream;  /**< redirect all score to file */

/**
 * interrupt handler inorder to free resources in case of ctrl-c. 
 */
void my_handler(int s){
    /* pcap related freeing of resources */
    pcap_freecode(fp);
    pcap_breakloop(handle);
    pcap_close(handle);
    cout << "[*] Total Packet:"<< packet_count << endl;
    cout << "[*] Total Time:"<< total_time<<endl;
    cout << "[*] Avg. Time/Packet: "<<total_time/(double)packet_count << endl;

    if(global_param.output_flag == 1)
    {
        fileStream.flush();
        fileStream.close();
    }
        
    /* free sdhash related resources */
    if(global_param.sdhash_flag == 1)
    {
        delete set_original;
        delete reverse_set_original;
    }

    /* free mrsh-v2 related resources */
    if(global_param.mrsh_flag == 1)
    {
        free(mode); //globally initialized once
        fingerprintList_destroy(fpl);
        //cout << "[*] Average mrsh score: " << mrsh_total_score/(double)packet_count << endl;
        //destroy_bloom_arr(bloom_arr);
    }

    /* statistics related to the present session */
    if(global_param.statistics_flag == 1){
        cout<<"\n[*]Average Packet Length:"<< ((float)total_packet_size/(float)packet_count)
            <<"\n[*]Average Header Length:"<< ((float)total_header_length/(float)packet_count)
            <<"\n[*]Average Payload Length:"<< ((float)total_payload_length/(float)packet_count)
            <<"\n[*]Average IP header Length:"<< ((float)total_ip_header_size/(float)packet_count)
            <<"\n[*]Average Tcp Header Length:"<< ((float)total_tcp_header_size/(float)packet_count)
            <<"\n[*]Minimum Packet Length:"<<min_packet_size
            <<"\n[*]Maximum Packet Length:"<<max_packet_size
            <<endl;
    }
    exit(EXIT_SUCCESS);
}

/** 
 * to get the statistics of the present session of traffic analysis. 
 * @param header struct pcap packet header 
 * @param packet unsigned char pointer of pcap packet 
 */ 
void packet_statistics(const struct pcap_pkthdr *header, const u_char *packet)
{
    const struct sniff_ethernet *ethernet; /**< pointing to ethernet header*/ 
    const struct sniff_ip *ip;        /**< pointing to ip header*/ 
    const struct sniff_tcp *tcp;      /**< pointing to tcp header */
    const unsigned char *payload;     /**< pointing to payload */                           

    size_t size_ip_header;            /**< ip header length */
    size_t size_tcp_header;           /**< tcp header length */

    size_t size_packet = header->len; /**< total size of the packet */
    size_t size_ip;                   /**< size of packet ip header onwards */
    size_t size_tcp;                  /**< size of packet tcp header onwards */
    size_t size_payload;              /**< size of payload of the packet */

    size_t header_length; /**< header length including ethernet, ip and tcp */

    /*ethernet*/
    ethernet = (struct sniff_ethernet*)(packet);

    /*IP */
    ip = (struct sniff_ip*)(packet + SIZE_ETHERNET);
    size_ip_header = IP_HL(ip)*4;
    size_ip = ntohs(ip->ip_len);

    /*TCP */
    tcp = (struct sniff_tcp*)(packet + SIZE_ETHERNET + size_ip_header);
    size_tcp_header = TH_OFF(tcp)*4;

    /* define/compute tcp payload (segment) offset */
    payload = (unsigned char *)(packet + SIZE_ETHERNET + size_ip_header + size_tcp_header);
    size_payload = size_ip - (size_ip_header + size_tcp_header);
    size_tcp = size_tcp_header + size_payload;

    header_length = 14 + size_ip_header + size_tcp_header;

    /*get total lengths*/
    total_packet_size +=size_packet;
    total_header_length += header_length;
    total_payload_length +=size_payload;
    total_ip_header_size +=size_ip_header;
    total_tcp_header_size +=  size_tcp_header;

    max_packet_size = size_packet>max_packet_size? size_packet:max_packet_size;
    min_packet_size = size_packet<min_packet_size? size_packet:min_packet_size;
}

/**
 * to reverse a given packet byte wise
 * @param buff unsigned char pointer containing reversed packet
 * @param arr unsigned char pointer which is to be reversed
 * @param length integer for the length of the packet under consideration
 */
void reverse_array(unsigned char *buff, const unsigned char *arr, int length){
    int i;
    for(i = 0;i<length; i++)
    {
        buff[length -i-1] = arr[i];
    }
}

/*
 * performs match using sdhash, against the original file 
 * @param packet unsigned char pointer for the network packet
 * @param len lenght of the packet under consideration
 * @param packet_type type of packet being analysed: whole_packet, ip, tcp, payload
 * @see sdhash_got_packet()
 * @return returns the result set after performing comparison using sdhash. 
 */ 
string sdhash_perform_match(const u_char *packet, const size_t len, string packet_type)
{
    std::string resultlist;  /**< result returned from sdhash */

    //construct a set for the current packet to perform match.
    sdbf *sdbf_obj = new sdbf(packet_type.c_str(), (char*) packet, SDHASH_BLOCK_SIZE, len);
    sdbf_set *sdbf_query = new sdbf_set();
    sdbf_query->add(sdbf_obj);
    resultlist = set_original->compare_to(sdbf_query, SDHASH_THRESHOLD, SDHASH_SAMPLE_SIZE);

    //freeing resources
    delete sdbf_obj;
    delete sdbf_query;
    return resultlist;
}

/**
 * called from pcap to perform match using sdhash.
 * @param header pcap packet header
 * @param packet unsigned char pointer to the packet
 * @see sdhash_perform_match()
 * @see got_packet()
 */ 
void sdhash_got_packet(const struct pcap_pkthdr *header, const u_char *packet)
{
    try{
        START;
        /*getting the header information*/
        const struct sniff_ethernet *ethernet;    /**< pointing to ethernet header*/ 
        const struct sniff_ip *ip;                /**< pointing to ip header*/ 
        const struct sniff_tcp *tcp;              /**< pointing to tcp header */   
        const unsigned char *payload;             /**< pointing to payload */ 

        size_t size_ip_header;                  /**< ip header length */
        size_t size_tcp_header;                 /**< tcp header length */

        size_t size_packet = header->len;      /**< total size of the packet */
        size_t size_ip;                        /**< size of packet ip header onwards */
        size_t size_tcp;                       /**< size of packet tcp header onwards */ 
        size_t size_payload;                   /**< size of payload of the packet */ 

        /*ethernet*/
        ethernet = (struct sniff_ethernet*)(packet);

        /*ip*/
        ip = (struct sniff_ip*)(packet + SIZE_ETHERNET);       //compute ip header offset 
        size_ip_header = IP_HL(ip)*4;
        size_ip = ntohs(ip->ip_len);

        /*TCP*/
        tcp = (struct sniff_tcp*)(packet + SIZE_ETHERNET + size_ip_header);
        size_tcp_header = TH_OFF(tcp)*4;

        /*Payload*/
        payload = (unsigned char *)(packet + SIZE_ETHERNET + size_ip_header + size_tcp_header);
        size_payload = size_ip - (size_ip_header + size_tcp_header);
        size_tcp = size_tcp_header + size_payload;

        /*strings containing respective results*/
        /*std::string resultlist_pkt;
        std::string resultlist_ip;
        std::string resultlist_tcp;
        */
        std::string resultlist_payload;

        /*WHOLE PACKET*/
        if(global_param.verbose_flag== 1)
        {
            cout << "[+] Starting match of Whole packet \n";
        }

        resultlist_pkt = sdhash_perform_match(packet, size_packet, "whole_packet");
        /*END*/

        /*IP PACKET */
        resultlist_ip = sdhash_perform_match((unsigned char*)ip, size_ip, "ip_packet");

        /*TCP PACKET*/
        resultlist_tcp = sdhash_perform_match((unsigned char*)tcp, size_tcp, "tcp_packet");

        /*PAYLOAD PACKET*/
        resultlist_payload = sdhash_perform_match((unsigned char*)payload, size_payload, "payload_packet");

        /*stop timer for time calculation */
        STOP;
       
        if(global_param.output_flag == 1){
            fileStream << "[*] Packet number:"<<packet_count<<" \n";
            fileStream << "[*] Total packet: "<<resultlist_pkt ;
            fileStream << "[*] IP packet: "<<resultlist_ip ; 
            fileStream << "[*] Tcp packet: "<<resultlist_tcp ;   
            
            fileStream << "[*] Payload packet:"<<resultlist_payload ;
            //fileStream << resultlist_payload;
            fileStream << "[*] Time:"<< TIME << endl <<endl; 

        }
        else{
            cout << "[*] Packet number:"<<packet_count<<" \n";
            cout << "[*] Total packet: "<<resultlist_pkt ;
            cout << "[*] IP packet: "<<resultlist_ip ; 
            cout << "[*] TCP packet: "<<resultlist_tcp ;   
            
            cout << "[*] Payload packet:"<<resultlist_payload ;
            cout << resultlist_payload;
            cout <<"[*] Time:"<< TIME<<endl<<endl; 
        }
        total_time +=TIME;
    }

    catch(exception& e){
        cerr << "[!] Exception while analysing packet using sdhash at packet number: " << packet_count << e.what() << endl;
    }

    catch(int param){
        cerr << "[!] Exception while analysing packet using sdhash at packet number: " << packet_count << param << endl;
    }
    catch(char param){
        cerr << "[!] Exception while analysing packet using sdhash at packet number: " << packet_count << param << endl;
    } 
    catch(...){
        cerr << "[!] Exception while analysing packet using sdhash at packet number: " << packet_count << endl;
    }
}

void init_mrsh_mode(){
    mode = (MODES *) malloc (sizeof(MODES)); 
    mode->compare = false;
    mode->gen_compare = false;
    mode->compareLists = false;
    mode->file_comparison = false;
    mode->helpmessage = false;
    mode->print = false;
    mode->threshold = 85;
    mode->recursive = false;
    mode->path_list_compare = false;

}


/**
 * called from pcap to perform match using mrsh
 * @param header pcap packet header
 * @param packet unsigned char pointer to the packet
 * @see mrsh_compare_packet()
 * @see got_packet()
 */
void mrsh_got_packet(const struct pcap_pkthdr *header, const u_char *packet)
{
    try{

        /*start counter for time */
        START;

        /*getting the header information*/
        const struct sniff_ethernet *ethernet;    /**< pointing to ethernet header*/ 
        const struct sniff_ip *ip;                /**< pointing to ip header*/ 
        const struct sniff_tcp *tcp;              /**< pointing to tcp header */   
        const unsigned char *payload;             /**< pointing to payload */ 

        size_t size_ip_header;                  /**< ip header length */
        size_t size_tcp_header;                 /**< tcp header length */

        size_t size_packet = header->len;      /**< total size of the packet */
        size_t size_ip;                        /**< size of packet ip header onwards */
        size_t size_tcp;                       /**< size of packet tcp header onwards */ 
        size_t size_payload;                   /**< size of payload of the packet */ 

        /*eth*/
        ethernet = (struct sniff_ethernet*)(packet);

        /*ip*/
        ip = (struct sniff_ip*)(packet + SIZE_ETHERNET);       //compute ip header offset 
        size_ip_header = IP_HL(ip)*4;
        size_ip = ntohs(ip->ip_len);
        string source_ip = inet_ntoa(ip->ip_src);
        string dest_ip = inet_ntoa(ip->ip_dst);
        u_short source_port = tcp->th_sport;
        u_short dest_port = tcp->th_dport;

        /*tcp*/
        tcp = (struct sniff_tcp*)(packet + SIZE_ETHERNET + size_ip_header);
        size_tcp_header = TH_OFF(tcp)*4;

        /*payload*/
        payload = (unsigned char *)(packet + SIZE_ETHERNET + size_ip_header + size_tcp_header);
        size_payload = size_ip -  (size_ip_header + size_tcp_header);
        size_tcp = size_tcp_header + size_payload;

        if(global_param.verbose_flag ==1)
        {
            cout << "[+] Strating match of whole packet \n";
        }

        /*whole packet*/
       /**
        cout<<"[*]Packet number:" << packet_count <<endl;
        FINGERPRINT *fp_eth = init_empty_fingerprint();
        strcpy(fp_eth->file_name, "whole_packet");
        hashPacketBuffer(fp_eth, packet, size_packet);
        cout<<"[*]total packet:";
        fingerprint_against_list_comparison(fpl, fp_eth);

        FINGERPRINT *fp_ip = init_empty_fingerprint();
        strcpy(fp_ip->file_name,"ip_packet");
        hashPacketBuffer(fp_ip,(const unsigned char*) ip, size_ip);
        cout<<"[*]IP packet:";
        fingerprint_against_list_comparison(fpl, fp_ip);
        
        FINGERPRINT *fp_tcp = init_empty_fingerprint();
        strcpy(fp_tcp->file_name,"tcp_packet");
        hashPacketBuffer(fp_tcp, (const unsigned char*) tcp, size_tcp);
        cout<<"[*]tcp packet:";
        fingerprint_against_list_comparison(fpl, fp_tcp);
        */

        char pkt_count_char[20];
        int_to_char(packet_count, pkt_count_char);
        /*PAYLOAD*/
        MATCH_LIST *match_list=initialize_match_list();
        FINGERPRINT *fp_payload = init_empty_fingerprint();
        strcpy(fp_payload->file_name,pkt_count_char);
        hashPacketBuffer(fp_payload,  payload, size_payload);
        fingerprint_against_list_comparison(fpl, fp_payload, match_list);

        /*stop timer */
        STOP;
        total_time+=TIME;
        
        MATCH_RESULT *node = match_list->head;
        while(node!=NULL)
        {
            printf("%s | %s | %.3i \n", node->filename1, node->filename2, node->score);
            node=node->next;
        }
        //MULTIPLE PACKET ANALYSIS
        /**
        if(global_param.mutliple_packet_analysis_flag == 1)
        {
            set<connection>::iterator it;
            connection conn_tmp(source_ip, dest_ip, source_port, dest_port);
            if(mrsh_multiple_packet_set.find(conn_tmp)!=mrsh_multiple_packet_set.end())
            {
                mrsh_multiple_packet_set.insert(conn_tmp);
            }
            else
            {
                if(match_list->total_length < MATCH_PER_PACKET)
                {
                    while(node!=NULL)
                    {
                        int result = conn_tmp.add_file_to_map(string(node->filename2));
                        if(result == FILE_DETECTED)
                        {
                            cout << node->filename2;
                            break;
                        }

                        else if(result==FILE_ADDED)
                        {
                            node=node->next;
                        }
                    }
                }
            }
        }
        else
        {
        while(node!=NULL)
        {
            printf("%s | %s | %.3i \n", node->filename1, node->filename2, node->score);
            node=node->next;
        }
        }*/

        destroy_match_list(match_list);
        fingerprint_destroy(fp_payload);    //free up the memory 


        //out_stream <<"[*] Time:"<<TIME<<"\n";
        //out_stream.flush();

        cout << endl;
    }
    catch (...)
    {
        cout <<"[!] Error in processing the packet by MRSH-v2. Packet Number: " << packet_count<<"\n";
    }
 
}
void write_packet_to_file(const struct pcap_pkthdr *header, const u_char *packet){

    const struct sniff_ethernet *ethernet;    /**< pointing to ethernet header*/ 
    const struct sniff_ip *ip;                /**< pointing to ip header*/ 
    const struct sniff_tcp *tcp;              /**< pointing to tcp header */   
    const unsigned char *payload;             /**< pointing to payload */ 

    size_t size_ip_header;                  /**< ip header length */
    size_t size_tcp_header;                 /**< tcp header length */

    size_t size_packet = header->len;      /**< total size of the packet */
    size_t size_ip;                        /**< size of packet ip header onwards */
    size_t size_tcp;                       /**< size of packet tcp header onwards */ 
    size_t size_payload;                   /**< size of payload of the packet */ 

    /*eth*/
    ethernet = (struct sniff_ethernet*)(packet);

    /*ip*/
    ip = (struct sniff_ip*)(packet + SIZE_ETHERNET);       //compute ip header offset 
    size_ip_header = IP_HL(ip)*4;
    size_ip = ntohs(ip->ip_len);

    /*tcp*/
    tcp = (struct sniff_tcp*)(packet + SIZE_ETHERNET + size_ip_header);
    size_tcp_header = TH_OFF(tcp)*4;

    /*payload*/
    payload = (unsigned char *)(packet + SIZE_ETHERNET + size_ip_header + size_tcp_header);
    size_payload = size_ip -  (size_ip_header + size_tcp_header);
    size_tcp = size_tcp_header + size_payload;

    write_eth_pkt_to_file((unsigned char *) packet, size_packet, (char *)global_param.write_packet_filename.c_str());
    write_ip_pkt_to_file((unsigned char *) ip, size_ip,(char *)global_param.write_packet_filename.c_str());
    write_tcp_pkt_to_file((unsigned char *) tcp, size_tcp,(char *)global_param.write_packet_filename.c_str());
    write_payload_pkt_to_file(payload, size_payload, (char *)global_param.write_packet_filename.c_str());


}

static void help_message(){
    //cout<< "Sniffer by Vikas Gupta" <<endl
     //   << "Copyright (C) 2013 " << endl;
    cout<<"Options:" << endl
        <<"       -h: For help message \n"
        <<"       -a: mr for mrsh-v2 and sd for sdhash \n"
        <<"       -d: Network interface to sniff on, lo for loopback \n"
        <<"       -l: File containing list of hashed files \n"
        <<"       -f: Specify filter, in Pcap filter format \n"
        <<"       -O: Read from offline packet dump in Pcap format\n"
        <<"       -w: Write network packets to file for offline use\n"
        <<"       -m: multiple packet anlaysis(not working) \n"
        <<"       -s: Get Statistics of the Session\n";
} 

/**
 * called by pcap library by default for each packet
 *
 */ 
void got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet){

    ++packet_count;
    /*call respective methods depending on mrsh-sdhash */

    if(global_param.write_packet_flag == 1)
    {
        write_packet_to_file(header, packet);
    }
    else{

        if(global_param.mrsh_flag == 1)
            mrsh_got_packet(header, packet);
        else if (global_param.sdhash_flag == 1)
            sdhash_got_packet(header, packet);
        else
            cout << "[*] No algorithm specified. Program exiting .. \n";

        if(global_param.statistics_flag == 1)
            packet_statistics(header, packet);
    }
}

int main(int argc, char *argv[]){
    char errbuf[PCAP_ERRBUF_SIZE];  /**< buffer for getting pcap errors */
    fp = (struct bpf_program *)malloc(sizeof(struct bpf_program)); /**< compilled filter expression for pcap */     

    struct pcap_pkthdr header; /**< pcap header */                         
    bpf_u_int32 mask;    /**< net mask of the device being sniffed for network traffic */                       
    bpf_u_int32 net;   /**< IP address of the device */ 

    const u_char *packet;  /**< pointer to the packet */
    int num_packets = -1;  /**< all packets till the end of file in offline mode and infinite in online mode */


    /* to handle ctrl-c interupts, and free up resources */
    struct sigaction sigIntHandler;
    sigIntHandler.sa_handler = my_handler;
    sigemptyset(&sigIntHandler.sa_mask);
    sigIntHandler.sa_flags = 0;
    sigaction(SIGINT, &sigIntHandler, NULL);

    opterr = 0;
    int c;

    /* if no arguments passed, exit */
    if(argc == 1)
    {
        help_message();
        exit(EXIT_SUCCESS);
    }

    while((c=getopt(argc, argv, "hvso:l:O:w:a:d:f:"))!=-1)
    {
        switch(c)
        {
            case 'h':  //help message
                help_message();
                exit(EXIT_SUCCESS);
                break;
            case 'v' :   //verbose mode
                //optimize it
                global_param.verbose_flag = 1;
                break;
            case 'o':   //output file for writing matching scores   
                global_param.output_flag = 1;
                global_param.output_file = optarg;
                fileStream.open(optarg,ios::out| ios::app);
                break;
            case 'l':       //input file containing list of hashed files     
                global_param.hash_list_flag =1 ;
                global_param.hash_list_name = optarg;
                break;
            case 's':       //get statistics for the session
                global_param.statistics_flag = 1;
                break;
            case 'a':       //algorithm to be used
                int cmp_score;
                if(strncmp(optarg, "mr", 2) == 0)
                {
                    global_param.mrsh_flag = 1;
                    global_param.sdhash_flag = 0;
                    fpl = init_empty_fingerprintList(); /**< array of mrsh bloom filters containing hash of all original files */

                }
                else if(strncmp(optarg, "sd", 2) == 0)
                {
                    global_param.sdhash_flag = 1;
                    global_param.mrsh_flag = 0;
                }
                else
                {
                    cout << "Please provide an algorithm to be used \n"
                        << "mr : mrsh-v2 \n"
                        << "sh : sdhash \n";
                    exit(EXIT_SUCCESS);       
                }
                break;
            case 'O':       //offline mode: give the filename of the pcap_dump file
                global_param.offline_flag = 1;
                global_param.dump_filename = optarg;
                break;
            case 'w':       //write packets: eth, ip, tcp, payload to a file
                global_param.write_packet_flag =1;
                global_param.write_packet_filename = optarg;
                const char *cstr ;
                cstr = global_param.write_packet_filename.c_str();
                //create_dir(cstr);
                break;
            case 'd': //device to sniff on
                global_param.dev = optarg;
                break;
            case 'f':  //filter expression
                global_param.filter = optarg;
                break;
            case 'm':
                global_param.mutliple_packet_analysis_flag = 1;
                break;
            default:
                help_message();
                exit(EXIT_SUCCESS);       
                break;
        }
    }

    
   if(global_param.mrsh_flag == 1)
   {
       FILE *hash_list =  fopen(global_param.hash_list_name.c_str(), "r");
       if(hash_list == NULL)
       {
           cout<< "[!] Unable to open/read the hash_list file \n"
               << "program exiting ... ";
           exit(EXIT_SUCCESS);
       }
       else
       {
           init_mrsh_mode(); //no need to initialize mode for each packet. 
           read_fingerprint_file(fpl, hash_list);
           cout << "[*] Number of hashes read: " <<fpl->size<< "\n";
       }
   }
   else if(global_param.sdhash_flag == 1)
   {
       try
       {
           set_original = new sdbf_set(global_param.hash_list_name.c_str());
           cout <<"[*] Number of hashes read:"<< set_original->size()<<"\n";
       }
       catch(...)
       {
           cout << "Sdhash was not able to read the hash list file \n"
               << "Program Exiting....";
           exit(EXIT_SUCCESS);
       }
   }
   /* find the properties for the device*/
   if (pcap_lookupnet(global_param.dev.c_str(), &net, &mask, errbuf) == -1){
       cerr << "Can't get netmask for device:"<< global_param.dev << endl;
       net = 0;
       mask = 0;
   }

   //OFFLINE OR ONLINE MODE
   if(global_param.offline_flag == 1){
       //FOR OFFLINE USAGE
       handle = pcap_open_offline(global_param.dump_filename.c_str(), errbuf);
   }
   else{
       //BUFSIZE: max number of bytes to capture
       /*open the session in the promisu_cuous mode */
       handle = pcap_open_live(global_param.dev.c_str(), BUFSIZ, PROMISC, TIMEOUT, errbuf);
    }
    cout<<">>>>>>>>>SCAN STARTED<<<<<<<<<<<<" << endl;
    if (handle==NULL)
    {
       cerr<<"[!]Couldn't open device: "<<global_param.dev << "becuase" << endl
           << errbuf << endl;
        exit(EXIT_SUCCESS);
    }
    if (pcap_compile(handle, fp, global_param.filter.c_str(), 0, net) == -1)
    {
        cerr<< "Couldn't parse filter: "<<global_param.filter <<endl<< "error:"<< pcap_geterr(handle);
       exit(EXIT_SUCCESS); 
    }

    /* apply the filter */
    if (pcap_setfilter(handle, fp) == -1){
        cerr<< "Couldn't parse filter:" << global_param.filter<<endl <<"error:" << pcap_geterr(handle);
       exit(EXIT_SUCCESS); 
    }

    int pcap_result;
    pcap_result= pcap_loop(handle, num_packets, got_packet, NULL);
    

    //CODE CLEANUP in case no interrupt , 1 is a hack
    my_handler(1);
} 
