#include <iostream>
#include <pcap.h>
#include <fstream>
#include <string>
#include <stdlib.h>
#include "boost/program_options.hpp"

#include "../header/timing.h"
#include "../header/config.h"
#include "../header/print_pkt.h"
#include "../header/util.h"
#include "../header/sdhash.h"

using namespace std;

extern long packet_count;
extern sniffer_parameters global_param;
extern sdbf_set *set_original, *reverse_set_original;
extern std::streambuf *buf;
extern std::ostream out_stream(buf);
extern double total_time;

string sdhash_perform_match(const u_char *packet, const size_t len, string packet_type)
{
    std::string resultlist;
    sdbf *sdbf_obj = new sdbf(packet_type.c_str(), (char*) packet, SDHASH_BLOCK_SIZE, len);
    sdbf_set *sdbf_query = new sdbf_set();
    sdbf_query->add(sdbf_obj);
    resultlist = set_original->compare_to(sdbf_query, SDHASH_THRESHOLD, SDHASH_SAMPLE_SIZE);
    delete sdbf_obj;
    delete sdbf_query;
    return resultlist;
}

void sdhash_got_packet(const struct pcap_pkthdr *header, const u_char *packet)
{
    try{
        packet_count++;
        START;
        /*getting the header information*/
        const struct sniff_ethernet *ethernet;              //the ethernet header
        const struct sniff_ip *ip;                          // ip header
        const struct sniff_tcp *tcp;                        //tcp header
        const unsigned char *payload;                                //packet payload

        size_t size_ip_header;
        size_t size_tcp_header;                                       // size of tcp header 
        
        size_t size_packet = header->len;
        size_t size_ip;
        size_t size_tcp;
        size_t size_payload;                                   //size of the payload sans tcp header

        ethernet = (struct sniff_ethernet*)(packet);

        ip = (struct sniff_ip*)(packet + SIZE_ETHERNET);       //compute ip header offset 
        size_ip_header = IP_HL(ip)*4;
        size_ip = ntohs(ip->ip_len);

        tcp = (struct sniff_tcp*)(packet + SIZE_ETHERNET + size_ip_header);
        size_tcp_header = TH_OFF(tcp)*4;
        size_tcp = size_tcp_header + size_payload;

        /* define/compute tcp payload (segment) offset */
        payload = (unsigned char *)(packet + SIZE_ETHERNET + size_ip_header + size_tcp_header);
        size_payload = size_ip - (size_ip_header + size_tcp_header);
        /* compute tcp payload (segment) size */

        /*results string for each layer*/
        std::string resultlist_pkt;
        std::string resultlist_ip;
        std::string resultlist_tcp;
        std::string resultlist_payload;

        /*WHOLE PACKET*/
        if(global_param.verbose_flag == 1)
        {
            cout << "[+] Starting match of Whole packet \n";
        }
        resultlist_pkt = sdhash_perform_match(packet, size_packet, "whole_packet");
        /*END*/

        /*IP PACKET */
        if(global_param.verbose_flag == 1)
        {
            cout << "[+] Starting match of ip packet \n";
        }
        resultlist_ip = sdhash_perform_match(packet, size_ip, "ip_packet");
        /*END*/

        /*TCP PACKET*/
        if(global_param.verbose_flag == 1)
        {
            cout << "[+] Starting match of tcp packet \n";
        }
        resultlist_tcp =  sdhash_perform_match((unsigned char*)tcp, size_tcp, "tcp_packet");
        /*END*/

        /*PAYLOAD PACKET*/
        if(global_param.verbose_flag== 1)
        {
            cout << "[+] Starting match of payload packet \n";
        }
        resultlist_payload = sdhash_perform_match((unsigned char*)payload, size_payload, "payload_packet");
        /*END*/

        /*Testing for reverse of packets*/
        std::string reverse_resultlist_ip;
        std::string reverse_resultlist_pkt;
        std::string reverse_resultlist_tcp;
        std::string reverse_resultlist_payload;
        if(global_param.reverse_flag == 1)
        {
            if(global_param.verbose_flag == 1)
            {
                cout << "[+] Starting match of reversed packet \n";
            }

            //packet
            unsigned char *buff = (unsigned char *)malloc((size_packet+1)*sizeof(char));
            reverse_array(buff, packet, size_packet);
            reverse_resultlist_pkt = sdhash_perform_match(buff, size_packet, SDHASH_BLOCK_SIZE);
            free(buff);

            //ip
            unsigned char *buff_ip = (unsigned char *)malloc((size_ip)*sizeof(char));
            reverse_array(buff_ip,(unsigned char *) ip,size_ip);
            reverse_resultlist_ip = sdhash_perform_match(buff_ip,size_ip, "reverse_ip_packet");
            free(buff_ip);

            //tcp
            unsigned char *buff_tcp = (unsigned char *)malloc((size_tcp)*sizeof(char));
            reverse_array(buff_tcp,(unsigned char*)tcp, size_tcp);
            reverse_resultlist_tcp =  sdhash_perform_match(buff_tcp, size_tcp, "reverse_tcp_packet");
            free(buff_tcp);

            //payload
            unsigned char *buff_payload = (unsigned char *)malloc((size_payload)*sizeof(char));
            reverse_array(buff_payload, payload, size_payload);
            reverse_resultlist_payload = sdhash_perform_match(buff_payload, size_payload, "reverse_payload_packet");
            free(buff_payload);
        }
        /*END*/ 

        /*Write packets to file*/
        if(global_param.write_packet_flag ==1)
        {
            if(global_param.verbose_flag == 1)
                cout << "[+] Starting writing network packet to file" <<endl;

            write_eth_pkt_to_file((unsigned char *)packet, size_packet);
            write_ip_pkt_to_file((unsigned char *)ip, size_ip);
            write_tcp_pkt_to_file((unsigned char*)tcp, size_tcp);
            write_payload_pkt_to_file(payload, size_payload);
        }
        /*END*/

        /*print output*/
        out_stream<< "[*]Packet number:"<<packet_count<<" \n";
        out_stream<< "[*]total packet: "<<resultlist_pkt ;
        out_stream<< "[*]IP packet: "<<resultlist_ip ; 
        out_stream<< "[*]tcp packet: "<<resultlist_tcp ;   
        out_stream<< "[*]payload packet:"<<resultlist_payload ;
        if(global_param.reverse_flag == 1)
        {
            out_stream<<reverse_resultlist_pkt;
            out_stream<<reverse_resultlist_ip;
            out_stream<<reverse_resultlist_tcp;
            out_stream<<reverse_resultlist_payload;
        }
        /*END*/

        STOP;
        out_stream<<"[*]Time:"<< TIME<<endl<<endl; 
        out_stream.flush();

        total_time +=TIME;
    }
    catch(...){
        cerr<<"[!] Exception while analysing packet using sdhash" <<endl;
    }

}

