#include "sdhash-cli.h"

#include <ctype.h>

// Global parameters
extern sdhash_parameters_t sdbf_sys;

/* 
 * Process command-line options:
 */
int process_opts( int argc, char **argv) {
    uint32_t i, opt_cnt=0;
    char opt;

    static struct option long_options[] = {
                {"heat-map",    no_argument, 0,'m'},
                {"warnings",no_argument, 0,'w'},
                {"hash-list",no_argument, 0,'f'},
                {"import",no_argument, 0,'i'},
                {"export",no_argument, 0,'e'},
                {"create-view",no_argument, 0,'v'},
                {"show-set-names",no_argument, 0,'n'},
                {"show-set-data",no_argument, 0,'d'},
                {"list",no_argument, 0,'l'},
                {"compare",no_argument, 0,'c'},
                {"query",no_argument, 0,'q'},
                {"hostname",required_argument, 0,'H'},
                {"port",required_argument, 0,'P'},
                {"block-size",required_argument, 0,'b'},
                {"threshold",required_argument, 0,'t'},
                {"sample-size",required_argument, 0,'s'},
                {"results",required_argument, 0,'r'},
                {"user",required_argument, 0,'u'},
				{"help",no_argument,0,0}
    };
    int option_index = 0;


    while( (opt = getopt_long (argc, argv, ":H:P:b:fildnvecqmwt:s:r:u:", long_options, &option_index)) != -1) {
        switch( opt) {
            case 'H':
                sdbf_sys.hostname = (char*)malloc(sizeof(char)*strlen(optarg)+1);
                strncpy(sdbf_sys.hostname,optarg,strlen(optarg)+1);
                break;
            case 'P':
                sdbf_sys.port = atoi(optarg);
                break;
            case 'c':
                sdbf_sys.options |= MODE_COMP;
                break;
            case 'q':
                sdbf_sys.options |= MODE_QUERY;
                break;
            case 'f':
                sdbf_sys.options |= MODE_INFILE;
                break;
            case 'i':
                sdbf_sys.options |= MODE_IMPORT;
                break;
            case 'l':
                sdbf_sys.options |= MODE_LIST;
                break;
            case 'n':
                sdbf_sys.options |= MODE_DISP;
                break;
            case 'd':
                sdbf_sys.options |= MODE_CONTENT;
                break;
            case 'v':
                sdbf_sys.options |= MODE_VIEW;
                break;
            case 'e':
                sdbf_sys.options |= MODE_EXPORT;
                break;
            case 'w':
                sdbf_sys.warnings = FLAG_ON;
                break;
            case 'b':
                sdbf_sys.dd_block_size = atoi( optarg);
                break;
            case 't':
                sdbf_sys.output_threshold = atoi( optarg);
                break;
            case 's':
                sdbf_sys.sample_size = atoi( optarg);
                break;
            case 'r':
                sdbf_sys.options |= MODE_RESULT;
				if (isdigit(optarg[0]))
					sdbf_sys.resultid = atoi(optarg);
				else {
					sdbf_sys.username = (char*)malloc(sizeof(char)*strlen(optarg)+1);
					strncpy(sdbf_sys.username,optarg,strlen(optarg)+1);
					sdbf_sys.resultid = 0;
				}
                break;
            case 'u':
                sdbf_sys.username = (char*)malloc(sizeof(char)*strlen(optarg)+1);
                strncpy(sdbf_sys.username,optarg,strlen(optarg)+1);
                break;
            case ':':
                fprintf( stderr, ">>> ERROR: Missing parameter for option -%c.\n", optopt);
                return -1;
                break;
            default:
                return -1;
        }
    }
    if (sdbf_sys.options == 0) {
        sdbf_sys.options |= MODE_HASH;
    }
    if( sdbf_sys.output_threshold < -1 || sdbf_sys.output_threshold > 100) {
        fprintf( stderr, "Error: invalid output threshhold (%d); resetting to 1.\n", sdbf_sys.output_threshold);
        sdbf_sys.output_threshold = 1;
    }
    return optind;
}

/*
 *  Show usage instructions
 */
void print_usage( char *version_info, char *command) {
    printf( "%s\n", version_info);
 	printf("    http://sdhash.org, license Apache v2.0\n");
    printf( "Usage: sdhash-cli <options> <files> \n");
	printf( "options:\n");
    printf( "  (none) <files>             hash files and save as new set.\n");
    printf( "  -f, --hash-list <file>     generate hashes from list of filenames\n");
    printf( "  -i, --import <file>        load new hash set from file\n");
    printf( "  -e, --export <N> <file>    saves hash set #N to file\n");
    printf( "  -l, --list                 lists available hash sets\n");
    printf( "  -d, --show-set-data <N>    displays hash set #N\n");
    printf( "  -n, --show-set-names <N>   displays filenames/sizes of hash set #N\n");
    printf( "  -c, --compare <N>          compare all pairs in set #N.\n");
    printf( "  -q, --query <N> <M>        compares hashes in set #N with set #M\n");
	printf( "  -r, --results <ID>|<name>  retrieves results by ID or name\n");
	printf( "  -u, --username <name>      sets username for results\n");
    printf( "  -v, --create-view <name> <N> <regex>\n    makes hash set <name> from set <N> using <regex>\n");
    printf( "  -H, --host <hostname>      defaults to localhost\n");
    printf( "  -P, --port <port>          default is 9090 (thrift)\n");
    printf( "  -b, --block-size <N>       hashes input files in N*KB blocks\n");
    printf( "  -t, --threshold <-1-100>   only show results >= threshold\n");
    printf( "  -s, --sample-size <1-16>   use N or fewer filters to match\n");
    printf( "  -w, --warnings             turn on warnings\n");
}


