
/**
 * sdhash-cli: Client for hashing server
 * author: Candice Quates
 */

#include <protocol/TBinaryProtocol.h>
#include <transport/TSocket.h>
#include <transport/TTransportUtils.h>

#include <boost/filesystem.hpp>

#include "sdhashsrv.h"

using namespace std;
using namespace apache::thrift;
using namespace apache::thrift::protocol;
using namespace apache::thrift::transport;

using namespace sdhash;

using namespace boost;


#include "sdhash-cli.h"

// Global parameters
sdhash_parameters_t sdbf_sys = {
    (char*)HOST,        // hostname
    PORT,        // port#
    0,                // dd block size off initially
    1,               // output_threshold
    FLAG_OFF,        // warnings
    0,              // sample size off
    0,        // default no options set
    -1,        // result ID off
    (char*)"default"        // default username
};

int main( int argc, char **argv) {
    uint32_t  i, j, k, file_cnt;
    int set1;
    char tempstring[]="/tmp/default-XXXXXX";
	boost::filesystem::path current_dir=boost::filesystem::initial_path();

//	cout << current_dir << endl;

    boost::shared_ptr<TTransport> socket(new TSocket("localhost", 9090));
    boost::shared_ptr<TTransport> transport(new TBufferedTransport(socket));
    boost::shared_ptr<TProtocol> protocol(new TBinaryProtocol(transport));
    sdhashsrvClient client(protocol);


    //set1 = set2 = -1;
    int file_start = process_opts( argc, argv);
    
    if( (argc < 2) || (file_start < 1)  ) {
        print_usage( (char*)VERSION_INFO, NULL);
        return -1;
    }
    file_cnt = argc-file_start;
    if (file_cnt == 0 && sdbf_sys.options != MODE_LIST) {
		if (sdbf_sys.options!=MODE_RESULT) {
			print_usage( (char*)VERSION_INFO, NULL);
			return -1;
		}
    }
    try {
        transport->open();
        client.ping();
        int i,fd;
        FILE *workfile;
        std::string result;
        uint32_t resultvalue = -2;
        switch (sdbf_sys.options) {
            case MODE_LIST:
                client.setList(result,false);
                break;
            case MODE_DISP:
                if (isdigit(argv[file_start][0])) {
                    client.displaySet(result,atoi(argv[file_start]));
                }
                break;
            case MODE_CONTENT:
                if (isdigit(argv[file_start][0])) {
                    client.displayContents(result,atoi(argv[file_start]));
                }
                break;
            case MODE_COMP:
                if (isdigit(argv[file_start][0])) {
					int32_t newid=client.createResultID(sdbf_sys.username);
                    client.compareAll(atoi(argv[file_start]),sdbf_sys.output_threshold,newid);
					std::cout << "Request name " << sdbf_sys.username <<", id: "<< newid << endl;
                }
                break;
            case MODE_QUERY:
                if (isdigit(argv[file_start][0]) && isdigit(argv[file_start+1][0])) {
					int32_t newid=client.createResultID(sdbf_sys.username);
                     client.compareTwo(atoi(argv[file_start]),atoi(argv[file_start+1])
                            ,sdbf_sys.output_threshold,sdbf_sys.sample_size,newid);
					std::cout << "Request name " << sdbf_sys.username <<", id: "<< newid << endl;
                }
                break;
            case MODE_IMPORT:
                resultvalue=client.loadSet(argv[file_start],-1);
                break;
            case MODE_INFILE:
                client.hashList(argv[file_start],sdbf_sys.dd_block_size,-1);
				result="Hashing in progress for "+(string)argv[file_start];
                break;
            case MODE_HASH:
                fd=mkstemp(tempstring);
                workfile=fdopen(fd,"w+");
                for (i=file_start ; i < argc; i++) {
					if (argv[i][0]=='/')
						fprintf(workfile, "%s\n",argv[i]);
					else
						fprintf(workfile,"%s/%s\n",current_dir.string().c_str(), argv[i]);
                }
                fflush(workfile);
                fclose(workfile);
                chmod(tempstring,S_IRUSR|S_IROTH);
                client.hashList(tempstring,sdbf_sys.dd_block_size, -1);
				result="Hashing in progress for "+(string)tempstring;
				//unlink(tempstring);
                break;
            case MODE_VIEW: 
                if (file_cnt == 3) {
                    if (isdigit(argv[file_start+1][0])) {
                        resultvalue=client.createView(argv[file_start],atoi(argv[file_start+1]),argv[file_start+2]);
                    }
                }
                break;
            case MODE_EXPORT:
                if (file_cnt == 2) {
                    if (isdigit(argv[file_start][0])) {
                        resultvalue=client.saveSet(atoi(argv[file_start]),argv[file_start+1]);
                        if (resultvalue==0) {
                            result = "Set saved";
                         }    
                    }
                }
                break;
			case MODE_RESULT: 
				if (sdbf_sys.resultid == 0) {
					client.displayResultsList(result,sdbf_sys.username,false);
				} else {
					client.displayResult(result,sdbf_sys.resultid);
				}
				break;
        }
        transport->close();
        if (result.length()==0) {
             if (resultvalue==-1) {
                cout << "ERROR: Command failed" << endl;
            } else if (resultvalue==-2) {
                //cout << "No results found" << endl;
            } else{
                cout << "Set "<< resultvalue << " created" << endl;
            }
        } else {
            cout << result << endl;
        }
    } catch (TException &tx) {
        printf("ERROR: %s\n", tx.what());
    }
    return 0;
}
