#include "sdbf_class.h"
#include "sdbf_defines.h"
#include "sdhash-srv.h"
#include "set_list.h"

#include <boost/regex.hpp>
#include <boost/filesystem.hpp>
#include <iostream>
#include <fstream>
#include <string>

#include <boost/thread/thread.hpp>

namespace fs = boost::filesystem;

#define CHUNK_SIZE 128*MB
 
// State
//static pthread_t *thread_pooll = NULL; // file-local thread pool

//static pthread_mutex_t hash_mutex = PTHREAD_MUTEX_INITIALIZER;


// NOT in sdbf class
/** 
	thread "process" function for file-by-file hashing
*/

void
*thread_sdbf_hashfile( void *task_param) {
    filehash_task_t *task = (filehash_task_t *)task_param;
    struct stat file_stat;
    int32_t chunks = 0,csize = 0;
    ifstream *is = new ifstream();

    int i,j;
    for( i=task->tid; i<task->file_count; i+=task->tcount) {
        if (stat(task->filenames[i],&file_stat))
            continue;
        is->open(task->filenames[i], ios::binary);
		try {
			class sdbf *sdbfm = new sdbf(task->filenames[i],is,0,file_stat.st_size);
			task->addset->add(sdbfm);
		} catch (int e) {
			if (e==-2)
			   exit(-2);
		}
        is->close();
    }
}

/**
 * Compute SD for a list of files & add them to a new set.
 * Not block-wise.
 
 */
void
sdbf_hash_files( char **filenames, uint32_t file_count, uint32_t thread_cnt,sdbf_set *addto ) {
    int32_t i, t, j;
    //sdbf_set *myset = new sdbf_set();
    struct stat file_stat;
    int32_t chunks = 0,csize = 0;
    ifstream *is = new ifstream();

    // Sequential implementation
    if( thread_cnt == 1) {
        for( i=0; i<file_count; i++) {
            if (stat(filenames[i],&file_stat))
                continue;
            is->open(filenames[i], ios::binary);
            try {
				//if (sdbf_sys.verbose) 
					//cerr << "sdhash: digesting file " << filenames[i] << endl;
                class sdbf *sdbfm = new sdbf(filenames[i],is,0,file_stat.st_size);
                if (!addto) {
                            cout << sdbfm ;
                    delete sdbfm;
                } else 
                    addto->add(sdbfm);
            } catch (int e) {
                if (e==-2)
                   exit(-2);
               // if (e==-3 && sdbf_sys.warnings) {
					//cerr << "Input file too small for processing: "<< filenames[i]<< endl;
                //}
            }
            is->close();
        }
    // Threaded implementation
    } else {
		boost::thread *thread_pooll[MAX_THREADS];
        filehash_task_t *tasks = (filehash_task_t *) alloc_check( ALLOC_ZERO, thread_cnt*sizeof( filehash_task_t), "sdbf_hash_files", "tasks", ERROR_EXIT);
        for( t=0; t<thread_cnt; t++) {
            tasks[t].tid = t;
            tasks[t].tcount = thread_cnt;
            tasks[t].filenames = filenames;
            tasks[t].file_count = file_count;
            tasks[t].addset = addto;
			thread_pooll[t] = new boost::thread(thread_sdbf_hashfile,tasks+t);
        }
        for( t=0; t<thread_cnt; t++) {
            thread_pooll[t]->join();
        }
        for( t=0; t<thread_cnt; t++) {
            delete thread_pooll[t];
        }
		free(tasks);
    // End threading
    }
    delete is;
}

/**
 * Compute SD for a filename containing a list of files & add them to a new set.
 */
sdbf_set*
hash_list(char *listfn, uint32_t dd_block_size,uint32_t thread_cnt) {
	sdbf_set *set1 = new sdbf_set();
	int i;
	struct stat stat_res;
    std::vector<string> small;
    std::vector<string> large;

	if( stat(listfn, &stat_res) != 0) {
		fprintf( stderr, "ERROR: Could not access input file \"%s\". Exiting.\n",listfn);
		return set1; 
	}
	processed_file_t *mlist=process_file(listfn, 1, 0);
	if (!mlist) {
		cerr << "sdhash: ERROR: Could not access input file "<< listfn << endl;
		return set1;
	}
	std::istringstream fromfile((char*)mlist->buffer);
	std::string fname;
	while (std::getline(fromfile,fname)) {
		if (fs::is_regular_file(fname)) {
			if (fs::file_size(fname) < 16*MB) {
                    small.push_back(fname);
			} else {
				large.push_back(fname);
			}
		}
	}
	int smallct=small.size();
	int largect=large.size();

	if (smallct > 0) {
        char **smalllist=(char **)alloc_check(ALLOC_ONLY,smallct*sizeof(char*),"main", "filename list", ERROR_EXIT);
        for (i=0; i < smallct ; i++) {
            smalllist[i]=(char*)alloc_check(ALLOC_ONLY,small[i].length()+1, "main", "filename", ERROR_EXIT);
            strncpy(smalllist[i],small[i].c_str(),small[i].length()+1);
        }
        if (dd_block_size < 1 )  {
			//delete set1;
			sdbf_hash_files( smalllist, smallct,thread_cnt,set1);
        } else {
            set1->add_and_hash( smalllist, smallct, dd_block_size*KB,128*MB);
        }
    }
    if (largect > 0) {
        char **largelist=(char **)alloc_check(ALLOC_ONLY,largect*sizeof(char*),"main", "filename list", ERROR_EXIT);
        for (i=0; i < largect ; i++) {
            largelist[i]=(char*)alloc_check(ALLOC_ONLY,large[i].length()+1, "main", "filename", ERROR_EXIT);
            strncpy(largelist[i],large[i].c_str(),large[i].length()+1);
        }
		if (dd_block_size < 1) {
			set1->add_and_hash(largelist, largect, 16*KB, 128*MB);
		} else {
			set1->add_and_hash(largelist, largect, dd_block_size*KB, 128*MB);
		}
       
    }

	return set1;
}

/**
 * Compute SD for a vector containing a list of files & add them to a new set.
 */
sdbf_set*
hash_stringlist(const std::vector<std::string> & filenames, uint32_t dd_block_size,uint32_t thread_cnt) {
	sdbf_set *set1 = new sdbf_set();
	int i;
	struct stat stat_res;
    std::vector<string> small;
    std::vector<string> large;
	for (vector<string>::const_iterator it=filenames.begin(); it < filenames.end(); it++) {
		if (fs::is_regular_file(*it)) {
			if (fs::file_size(*it) < 16*MB) {
                    small.push_back(*it);
			} else {
				large.push_back(*it);
			}
		}
	}
	int smallct=small.size();
	int largect=large.size();

	if (smallct > 0) {
        char **smalllist=(char **)alloc_check(ALLOC_ONLY,smallct*sizeof(char*),"main", "filename list", ERROR_EXIT);
        for (i=0; i < smallct ; i++) {
            smalllist[i]=(char*)alloc_check(ALLOC_ONLY,small[i].length()+1, "main", "filename", ERROR_EXIT);
            strncpy(smalllist[i],small[i].c_str(),small[i].length()+1);
        }
        if (dd_block_size < 1 )  {
			//delete set1;
			sdbf_hash_files( smalllist, smallct,thread_cnt,set1);
        } else {
            set1->add_and_hash( smalllist, smallct, dd_block_size*KB,128*MB);
        }
    }
    if (largect > 0) {
        char **largelist=(char **)alloc_check(ALLOC_ONLY,largect*sizeof(char*),"main", "filename list", ERROR_EXIT);
        for (i=0; i < largect ; i++) {
            largelist[i]=(char*)alloc_check(ALLOC_ONLY,large[i].length()+1, "main", "filename", ERROR_EXIT);
            strncpy(largelist[i],large[i].c_str(),large[i].length()+1);
        }
		if (dd_block_size < 1) {
			set1->add_and_hash(largelist, largect, 16*KB, 128*MB);
		} else {
			set1->add_and_hash(largelist, largect, dd_block_size*KB, 128*MB);
		}
       
    }

	return set1;
}